@@ -344,6 +344,8 @@ sub request_url {
         $req .= "/" . $self->_escape($self->_null_escape($request->get("value")));
         $req .= $self->_extract_path($request->get("path"));
     } elsif ($type eq LIST) {
+        # The (URI escaped) colon after the <domain> must be transformed into a slash
+        $req =~ s|%3A|/|i;
         $req .= $self->_extract_path($request->get("path"));
     } elsif ($type eq EXEC) {
         $req .= "/" . $self->_escape($request->get("operation"));
